function [dico,freq,nadded]=add_atoms(dico,freq,repcand,repfreq,mumax,M)

% syntax: [dico,freq,nadded]=add_atoms(dico,freq,repcand,repfreq,mumax,M)
%
% add promising candidates to a dictionary
%
% input:
% dico... d x K dictionary matrix 
% freq... m x K vector of usage scores for each dictionary atom
% repcand... d x L vector of candidate atoms to add
% repfreq... 1 x L vector of usage score of replacement candidates
%
% mumax... maximally allowed coherence between atoms - default 1./sqrt(2)
% M... value for newly added atoms - default 1

% output:
% dico... d x K dictionary with added atoms
% freq... m x K vector of usage score of old and new atoms
% nadded... number of added atoms
%
% Karin Schnass 28.05.18

%%%% preparations and basic safeguarding
if(nargin < 2)
    disp('[dico,freq,repcand,repfreq,nrep]=replace_coherent(dico,repcand,freq,repfreq,mumax,mode)');
    return;
end

[d,K]=size(dico);
[dl,L]=size(repcand);

nadded = 0;

if nargin < 4
    return;
end

if nargin < 5
    mumax = 1/sqrt(2);
end

if nargin < 6
    M=1;
end
    
[m, Kf] = size(freq);
[mr,Lf] = size(repfreq);

if Kf ~= K || dl~=d || Lf~=L || mr>1
    disp('check size of input matrices, >> help add_atoms');
    return;
end
%%% end of preparations

[repfreq,repind] = sort(repfreq,'descend');
repcand = repcand(:,repind);
num_promise = length(find(repfreq>1));

for ell = 1:num_promise 
    rcoh=max(abs(dico'*repcand(:,ell)));
    %%% check coherence
    if rcoh < mumax 
        dico=[dico,repcand(:,ell)];
        freq=[freq, M*ones(m,1)];
        nadded=nadded+1;
    end
end

