function [dico,freq,npruned]=prune_unused(dico,freq,threshold,maxprune)

% syntax: [pdico,pfreq,nprune]=prune_unused(dico,freq,threshold)
%
% prune unused atoms in a dictionary
%
% input:
% dico...   d x K dictionary matrix with unused atoms to prune
% freq...   m x K matrix counting how often/strongly atoms 
%               have been used in the last m iterations
% threshold... cutoff threshold (default: 1)
% maxprune...  maximal number of pruned atoms 
%                       (default: min(round(d/5),floor(K/2)) )
%
% output:
% dico... d x K-npruned dictionary 
% freq... m x K-npruned counter
% npruned... number of atoms that have been pruned
%
% Karin Schnass 28.05.18

%%%% preparations and basic safeguarding
if(nargin < 2)
    disp('[pdico,pfreq,nprune]=prune_unused(dico,freq,threshold)');
    return;
end

[d,K] = size(dico);
[m,Kf]= size(freq);

if Kf ~= K
    disp('mismatching matrix sizes')
    pdico = dico;
    pfreq = freq;
    npruned = 0;
    return
end

if K == 1
    pdico = dico;
    pfreq = freq;
    npruned = 0;
    return;
end

if nargin < 3
    threshold = 1;
end

if nargin < 4
    maxprune = min(round(d/5),floor(K/2)); 
end
%%% end of preparations


%%% prune unused atoms 

%%% compare maximal score in the last m runs to threshold
if m > 1
    score = max(freq);
    %%% alternative: score = mean(freq);    
else
    score = freq;
end
notused = find(score<threshold);
%%% if more than maximal number of atoms are below
%%% delete those with smallest score
if length(notused) > maxprune
    [~,I]=sort(score,'ascend');
    notused = I(1:maxprune);
end
dico(:,notused)=[];
freq(:,notused)=[];
npruned = length(notused);
