function [dico,freq,nrep,repcand,repfreq]=replace_unused(dico,repcand,freq,repfreq,threshold)

% syntax: [dico,freq,repcand,repfreq,nrep]=replace_unused(dico,freq,repcand,repfreq,threshold)
%
% replace unused atoms in a dictionary
%
% input:
% dico... d x K dictionary matrix with unused atoms to replace
% repcand... d x L matrix of replacement candidates 
% freq... m x K matrix of usage scores for each dictionary atom
% repfreq... 1 x L vector of usage score of replacement candidates
% threshold... replace atoms with average usage score below threshold
%               as long as there are candidates with better score                         

% output:
% dico... d x K dictionary matrix with replaced atoms
% freq... m x K matrix of usages score of old and new atoms
% nrep... number of replaced atoms
% repcand... d x l matrix of left over replacement candidates                      
% repfreq... 1 x l vector of usage score of left over candidates
%
% Karin Schnass 28.05.18

%%%% preparations and basic safeguarding
if(nargin < 3)
    disp('[dico,freq,repcand,repfreq,nrep]=replace_unused(dico,repcand,freq,repfreq,mumax,mode)');
    return;
end

[d,K]=size(dico);
[dl,L]=size(repcand);

if nargin < 4
    repfreq = ones(1,L);
end

if nargin < 5
    threshold = 1;
end
    
[m, Kf] = size(freq);
[mr,Lf] = size(repfreq);

if Kf ~= K || dl~=d || Lf~=L || mr>1
    disp('check size of input matrices, >> help replace_unused');
    nrep = 0;
    return;
end
%%% end of preparations

nrep = 0;

%%% sort candidates by usefulness score
[repfreq,repind] = sort(repfreq,'descend');
repcand = repcand(:,repind);

%%%%%% replace atoms used less than thresh times
if m > 1
    [minfreq,minind] = min(mean(freq,1));
else
    [minfreq,minind] = min(freq);
end

%%%% compare candidate to average contribution in last runs
while ~isempty(repfreq)
    if minfreq < min(threshold,repfreq(1)) 
        dico(:,minind) = repcand(:,1);
        freq(:,minind) = repfreq(1)*ones(m,1);
        repfreq(1)=[];
        repcand(:,1)=[];
        if m > 1
            [minfreq,minind] = min(mean(freq,1));
        else
            [minfreq,minind] = min(freq);
        end
        nrep = nrep+1;
    else 
        repfreq=[];
        repcand=[];
    end
end
