
%%%%%%%%% settings %%%%%%%%%%%%%%%%%%%%%%%%%
rng(42);

d = 32;               %%% Dirac - Hadamard if exists, else Dirac - DCT
S = 2;                %%% sparsity level
K = d*1.5;            %%% dictionary size
b = 0.1;              %%% max. decay of sparse coefficients c_i=(1-b)^i
rho = 0.25/sqrt(d);   %%% noise parameter

inittype = 'r';       %%% 'r' ... random, 
                      %%% 'b' ... bad (original with L doubles & L 1:1)
L = 8;                %%% for stable failure choose S*L << d and L=2^l
                       
runs = 10;            %%% number of trials
maxit = 25;           %%% iterations per trial
N = 20000;            %%% signals per iteration

%%%% end of settings %%%%%%%%%%%%%%%%%%%%%%%

%%%% preparations
if inittype == 'b'
    runs = 1;
    Lhad_exists = (round(log2(L)) == log2(L))...
        || (L>11 && round(log2(L/12)) == log2(L/12))...
            || (L>19 && round(log2(L/20)) == log2(L/20));
else
    dinits = randn(d,K*runs);
end

dhad_exists = (round(log2(d)) == log2(d))...
        || (d>11 && round(log2(d/12)) == log2(d/12))...
            || (L>19 && round(log2(d/20)) == log2(d/20));
if dhad_exists
    dico = [eye(d),hadamard(d)/sqrt(d)]; 
else
    dico = [eye(d),idct(eye(d))]; 
end
K = min(K,2*d);
dico = dico(:,1:K); 

rdicos = zeros(d,K*runs);
bestipr = zeros(K,runs);
bestrpi = zeros(K,runs);
%%%%%% end of preparations

%%%% start trials 
for run = 1:runs
    
    %%% initialise dictionary
    if inittype == 'b'
        if Lhad_exists
            pert = hadamard(L)/sqrt(L-1);
        else
            pert = idct(eye(L));
        end
        pert = pert - diag(diag(pert));
        pert = pert*diag(1./sqrt(sum(pert.*pert)))/1.5;
        
        if K>=d+L
            scale = diag(dico(:,d-L+1:d)'*dico(:,d+1:d+L));
            halfatoms = dico(:,d-L+1:d)+dico(:,d+1:d+L)*diag(sign(scale));
            doubles = dico(:,1:L)*([eye(L)+pert,eye(L)-pert]);
            baddico = [doubles,dico(:,L+1:d-L),halfatoms,dico(:,d+L+1:K)];
        else
            halfatoms = dico(:,d-L+1:d)+dico(:,d-2*L+1:d-L);
            doubles = dico(:,1:L)*([eye(L)+pert,eye(L)-pert]);
            baddico = [doubles,dico(:,L+1:d-2*L),halfatoms];
        end
        dinit = baddico;
    else
        dinit = dinits(:,K*(run-1)+1: K*run);
    end
    dinit = dinit*diag(1./sqrt(sum(dinit.*dinit)));
    rdico = dinit;
    %%%% end of initialisation
    
    imagesc(abs(aligndico(rdico,dico)'*dico));
    coh = max(max(abs(rdico'*rdico-eye(K))));
    condi = cond(rdico);
    title(['init, mu = ',num2str(coh), ', cond = ',num2str(condi)]);
    drawnow;
    
    %%% run ITKrM
    for it=1:maxit
        Y=makesparsesig(dico,N,S,S,b,rho,1); 
        rdico = itkrm(Y,K,S,1,rdico); 
        imagesc(abs(aligndico(rdico,dico)'*dico)); 
        coh = max(max(abs(rdico'*rdico-eye(K))));
        condi = cond(rdico);
        title(['it: ',num2str(it),', mu = ',num2str(coh), ', cond = ',num2str(condi)]);
        drawnow; 
    end
    %%% end of ITKrm
    rdicos(:,K*(run-1)+1: K*run) = rdico; 
    bestipr(:,run)=max(abs(rdico'*dico));
    bestrpi(:,run)=max(abs(dico'*rdico));
end